@@ -14,7 +14,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.02';
+$VERSION = '1.04';
 
 sub ProcessAFCP($$);
 
@@ -27,22 +27,29 @@ an auxiliary technique to store IPTC information in images, but is
 incompatible with some file formats.
 
 ExifTool will read and write (but not create) AFCP IPTC information in JPEG
-images, but not other image formats.
+and TIFF images.
     },
     IPTC => { SubDirectory => { TagTable => 'Image::ExifTool::IPTC::Main' } },
     TEXT => 'Text',
     Nail => {
         Name => 'ThumbnailImage',
-        ValueConv => q{
-            my $img = substr($val, 18);
-            return $self->ValidateImage(\$img,$tag);
+        # (the specification allows for a variable amount of padding before
+        #  the image after a 10-byte header, so look for the JPEG SOI marker,
+        #  otherwise assume a fixed 8 bytes of padding)
+        RawConv => q{
+            pos($val) = 10;
+            my $start = ($val =~ /\xff\xd8\xff/g) ? pos($val) - 3 : 18;
+            my $img = substr($val, $start);
+            return $self->ValidateImage(\$img, $tag);
         },
     },
     PrVw => {
         Name => 'PreviewImage',
-        ValueConv => q{
-            my $img = substr($val, 18);
-            return $self->ValidateImage(\$img,$tag);
+        RawConv => q{
+            pos($val) = 10;
+            my $start = ($val =~ /\xff\xd8\xff/g) ? pos($val) - 3 : 18;
+            my $img = substr($val, $start);
+            return $self->ValidateImage(\$img, $tag);
         },
     },
 );
@@ -251,7 +258,7 @@ scanning for AFCP information.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
