@@ -12,9 +12,10 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.01';
+$VERSION = '1.02';
 
 sub ProcessHP($$$);
+sub ProcessTDHD($$$);
 
 # HP EXIF-format maker notes (or is it Vivitar?)
 %Image::ExifTool::HP::Main = (
@@ -110,6 +111,91 @@ sub ProcessHP($$$);
     },
 );
 
+# proprietary format TDHD data written by Photosmart R837 (ref PH)
+%Image::ExifTool::HP::TDHD = (
+    PROCESS_PROC => \&ProcessTDHD,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    NOTES => q{
+        These tags are extracted from the APP6 "TDHD" segment of Photosmart R837
+        JPEG images.  Many other unknown tags exist in is data, and can be seen with
+        the Unknown (-u) option.
+    },
+    # (all subdirectories except TDHD and LSLV are automatically recognized
+    # by their "type" word of 0x10001)
+    TDHD => {
+        Name => 'TDHD',
+        SubDirectory => { TagTable => 'Image::ExifTool::HP::TDHD' },
+    },
+    LSLV => {
+        Name => 'LSLV',
+        SubDirectory => { TagTable => 'Image::ExifTool::HP::TDHD' },
+    },
+    FWRV => 'FirmwareVersion',
+    CMSN => 'SerialNumber', # (unverified)
+    # LTEM - some temperature?
+);
+
+#------------------------------------------------------------------------------
+# Process HP APP6 TDHD metadata (ref PH)
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
+# Returns: 1 on success
+sub ProcessTDHD($$$)
+{
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my $dataPt = $$dirInfo{DataPt};
+    my $dataPos = $$dirInfo{DataPos};
+    my $pos = $$dirInfo{DirStart};
+    my $dirEnd = $pos + $$dirInfo{DirLen};
+    my $unknown = $exifTool->Options('Unknown') || $exifTool->Options('Verbose');
+    $exifTool->VerboseDir('TDHD', undef, $$dirInfo{DirLen});
+    SetByteOrder('II');
+    while ($pos + 12 < $dirEnd) {
+        my $tag = substr($$dataPt, $pos, 4);
+        my $type = Get32u($dataPt, $pos + 4);
+        my $size = Get32u($dataPt, $pos + 8);
+        $pos += 12;
+        last if $size < 0 or $pos + $size > $dirEnd;
+        if ($type == 0x10001) {
+            # this is a subdirectory containing more tags
+            my %dirInfo = (
+                DataPt   => $dataPt,
+                DataPos  => $dataPos,
+                DirStart => $pos,
+                DirLen   => $size,
+            );
+            $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
+        } else {
+            if (not $$tagTablePtr{$tag} and $unknown) {
+                my $name = $tag;
+                $name =~ tr/-_A-Za-z0-9//dc;    # remove invalid characters
+                my %tagInfo = (
+                    Name => "HP_TDHD_$name",
+                    Unknown => 1,
+                );
+                # guess format based on data size
+                if ($size == 1) {
+                    $tagInfo{Format} = 'int8u';
+                } elsif ($size == 2) {
+                    $tagInfo{Format} = 'int16u';
+                } elsif ($size == 4) {
+                    $tagInfo{Format} = 'int32s';
+                } elsif ($size > 80) {
+                    $tagInfo{Binary} = 1;
+                }
+                Image::ExifTool::AddTagToTable($tagTablePtr, $tag, \%tagInfo);
+            }
+            $exifTool->HandleTag($tagTablePtr, $tag, undef,
+                DataPt  => $dataPt,
+                DataPos => $dataPos,
+                Start   => $pos,
+                Size    => $size,
+            );
+        }
+        $pos += $size;
+    }
+    return 1;
+}
+
 #------------------------------------------------------------------------------
 # Process HP maker notes
 # Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
@@ -163,7 +249,7 @@ Hewlett-Packard maker notes.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
