@@ -15,6 +15,9 @@
 #               8) http://graphics.cs.uni-sb.de/NMM/dist-0.9.1/Docs/Doxygen/html/mmreg_8h-source.html
 #               9) http://developers.videolan.org/vlc/vlc/doc/doxygen/html/codecs_8h-source.html
 #              10) http://wiki.multimedia.cx/index.php?title=TwoCC
+#              11) Andreas Winter (SCLive) private communication
+#              12) http://abcavi.kibi.ru/infotags.htm
+#              13) http://tech.ebu.ch/docs/tech/tech3285.pdf
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::RIFF;
@@ -23,7 +26,9 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.16';
+$VERSION = '1.28';
+
+sub ConvertTimecode($);
 
 # type of current stream
 $Image::ExifTool::RIFF::streamType = '';
@@ -279,17 +284,21 @@ $Image::ExifTool::RIFF::streamType = '';
 %Image::ExifTool::RIFF::Main = (
     PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessChunks,
     NOTES => q{
-        Windows WAV and AVI files are RIFF format files.  According to the EXIF
-        specification, Meta information is embedded in two types of RIFF C<LIST>
-        chunks: C<INFO> and C<exif>, and information about the audio content is
-        stored in the C<fmt > chunk.  As well as this information, some video
-        information and proprietary manufacturer-specific information is also
-        extracted.
+        The RIFF container format is used various types of fines including WAV, AVI
+        and WEBP.  According to the EXIF specification, Meta information is embedded
+        in two types of RIFF C<LIST> chunks: C<INFO> and C<exif>, and information
+        about the audio content is stored in the C<fmt > chunk.  As well as this
+        information, some video information and proprietary manufacturer-specific
+        information is also extracted.
     },
    'fmt ' => {
         Name => 'AudioFormat',
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::AudioFormat' },
     },
+   'bext' => {
+        Name => 'BroadcastExtension',
+        SubDirectory => { TagTable => 'Image::ExifTool::RIFF::BroadcastExt' },
+    },
     LIST_INFO => {
         Name => 'Info',
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::Info' },
@@ -308,7 +317,18 @@ $Image::ExifTool::RIFF::streamType = '';
     },
     LIST_ncdt => { #PH (Nikon metadata)
         Name => 'NikonData',
-        SubDirectory => { TagTable => 'Image::ExifTool::RIFF::Nikon' },
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Nikon::AVI',
+            # define ProcessProc here so we don't need to load RIFF.pm from Nikon.pm
+            ProcessProc => \&Image::ExifTool::RIFF::ProcessChunks,
+        },
+    },
+    LIST_hydt => { #PH (Pentax metadata)
+        Name => 'PentaxData',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Pentax::AVI',
+            ProcessProc => \&Image::ExifTool::RIFF::ProcessChunks,
+        },
     },
     JUNK => [
         {
@@ -332,12 +352,20 @@ $Image::ExifTool::RIFF::streamType = '';
             Name => 'RicohJunk',
             # the Ricoh Caplio GX stores sub-chunks in here
             Condition => '$$valPt =~ /^ucmt/',
-            SubDirectory => { TagTable => 'Image::ExifTool::Ricoh::AVI' },
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Ricoh::AVI',
+                ProcessProc => \&Image::ExifTool::RIFF::ProcessChunks,
+            },
         },
         {
-            Name => 'UnknownJunk',
-            # try to interpret unknown junk as a string
-            RawConv => '$_=$val; /^[^\0-\x1f\x7f-\xff]+$/ ? $_ : undef',
+            Name => 'PentaxJunk', # (Optio RS1000)
+            Condition => '$$valPt =~ /^IIII\x01\0/',
+            SubDirectory => { TagTable => 'Image::ExifTool::Pentax::Junk' },
+        },
+        {
+            Name => 'TextJunk',
+            # try to interpret unknown junk as an ASCII string
+            RawConv => '$val =~ /^([^\0-\x1f\x7f-\xff]+)\0*$/ ? $1 : undef',
         }
     ],
     _PMX => { #PH (Adobe CS3 Bridge)
@@ -349,6 +377,10 @@ $Image::ExifTool::RIFF::streamType = '';
         Name => 'OldXMP',
         Binary => 1,
     },
+    olym => {
+        Name => 'Olym',
+        SubDirectory => { TagTable => 'Image::ExifTool::Olympus::WAV' },
+    },
 );
 
 # the maker notes used by some digital cameras
@@ -357,25 +389,6 @@ $Image::ExifTool::RIFF::streamType = '';
     GROUPS => { 2 => 'Audio' },
 );
 
-# Nikon metadata in AVI videos (PH)
-%Image::ExifTool::RIFF::Nikon = (
-    PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessChunks,
-    NOTES => 'Proprietry RIFF tags found in Nikon AVI videos.',
-    GROUPS => { 2 => 'Video' },
-    ncvr => {
-        Name => 'NikonVers',
-        SubDirectory => { TagTable => 'Image::ExifTool::Nikon::AVIVers' },
-    },
-    nctg => {
-        Name => 'NikonTags',
-        SubDirectory => { TagTable => 'Image::ExifTool::Nikon::AVITags' },
-    },
-    ncth => {
-        Name => 'ThumbnailImage',
-        Binary => 1,
-    },
-);
-
 # Format and Audio Stream Format chunk data
 %Image::ExifTool::RIFF::AudioFormat = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
@@ -401,10 +414,62 @@ $Image::ExifTool::RIFF::streamType = '';
     7 => 'BitsPerSample',
 );
 
+# Broadcast Audio Extension 'bext' information (ref 13)
+%Image::ExifTool::RIFF::BroadcastExt = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Audio' },
+    NOTES => q{
+        Information found in the Broadcast Audio Extension chunk (see
+        L<http://tech.ebu.ch/docs/tech/tech3285.pdf>).
+    },
+    0 => {
+        Name => 'Description',
+        Format => 'string[256]',
+    },
+    256 => {
+        Name => 'Originator',
+        Format => 'string[32]',
+    },
+    288 => {
+        Name => 'OriginatorReference',
+        Format => 'string[32]',
+    },
+    320 => {
+        Name => 'DateTimeOriginal',
+        Description => 'Date/Time Original',
+        Groups => { 2 => 'Time' },
+        Format => 'string[18]',
+        ValueConv => '$_=$val; tr/-/:/; s/^(\d{4}:\d{2}:\d{2})/$1 /; $_',
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
+    338 => {
+        Name => 'TimeReference',
+        Notes => 'first sample count since midnight',
+        Format => 'int32u[2]',
+        ValueConv => 'my @v=split(" ",$val); $v[0] + $v[1] * 4294967296',
+    },
+    346 => {
+        Name => 'BWFVersion',
+        Format => 'int16u',
+    },
+    # 348 - int8u[64] - SMPTE 330M UMID (Unique Material Identifier)
+    # 412 - int8u[190] - reserved
+    602 => {
+        Name => 'CodingHistory',
+        Format => 'string[$size-602]',
+    },
+);
+
 # Sub chunks of INFO LIST chunk
 %Image::ExifTool::RIFF::Info = (
     PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessChunks,
     GROUPS => { 2 => 'Audio' },
+    NOTES => q{
+        RIFF INFO tags found in WAV audio and AVI video files.  Tags which are part
+        of the EXIF 2.3 specification have an underlined Tag Name in the HTML
+        version of this documentation.  Other tags are found in AVI files generated
+        by some software.
+    },
     IARL => 'ArchivalLocation',
     IART => { Name => 'Artist',    Groups => { 2 => 'Author' } },
     ICMS => 'Commissioned',
@@ -437,12 +502,133 @@ $Image::ExifTool::RIFF::streamType = '';
     ISRC => 'Source',
     ISRF => 'SourceForm',
     ITCH => 'Technician',
+#
+# 3rd party tags
+#
+    # internet movie database (ref 12)
+    ISGN => 'SecondaryGenre',
+    IWRI => 'WrittenBy',
+    IPRO => 'ProducedBy',
+    ICNM => 'Cinematographer',
+    IPDS => 'ProductionDesigner',
+    IEDT => 'EditedBy',
+    ICDS => 'CostumeDesigner',
+    IMUS => 'MusicBy',
+    ISTD => 'ProductionStudio',
+    IDST => 'DistributedBy',
+    ICNT => 'Country',
+    ILNG => 'Language',
+    IRTD => 'Rating',
+    ISTR => 'Starring',
+    # MovieID (ref12)
+    TITL => 'Title',
+    DIRC => 'Directory',
+    YEAR => 'Year',
+    GENR => 'Genre',
+    COMM => 'Comments',
+    LANG => 'Language',
+    AGES => 'Rated',
+    STAR => 'Starring',
+    CODE => 'EncodedBy',
+    PRT1 => 'Part',
+    PRT2 => 'NumberOfParts',
+    # Morgan Multimedia INFO tags (ref 12)
+    IAS1 => 'FirstLanguage',
+    IAS2 => 'SecondLanguage',
+    IAS3 => 'ThirdLanguage',
+    IAS4 => 'FourthLanguage',
+    IAS5 => 'FifthLanguage',
+    IAS6 => 'SixthLanguage',
+    IAS7 => 'SeventhLanguage',
+    IAS8 => 'EighthLanguage',
+    IAS9 => 'NinthLanguage',
+    ICAS => 'DefaultAudioStream',
+    IBSU => 'BaseURL',
+    ILGU => 'LogoURL',
+    ILIU => 'LogoIconURL',
+    IWMU => 'WatermarkURL',
+    IMIU => 'MoreInfoURL',
+    IMBI => 'MoreInfoBannerImage',
+    IMBU => 'MoreInfoBannerURL',
+    IMIT => 'MoreInfoText',
+    # GSpot INFO tags (ref 12)
+    IENC => 'EncodedBy',
+    IRIP => 'RippedBy',
+    # Sound Forge Pro tags
+    DISP => 'SoundSchemeTitle',
+    TLEN => { Name => 'Length', ValueConv => '$val/1000', PrintConv => '"$val s"' },
+    TRCK => 'TrackNumber',
+    TURL => 'URL',
+    TVER => 'Version',
+    LOCA => 'Location',
+    TORG => 'Organization',
+    # Sony Vegas AVI tags, also used by SCLive and Adobe Premier (ref 11)
+    TAPE => {
+        Name => 'TapeName',
+        Groups => { 2 => 'Video' },
+    },
+    TCOD => {
+        Name => 'StartTimecode',
+        # this is the tape time code for the start of the video
+        Groups => { 2 => 'Video' },
+        ValueConv => '$val * 1e-7',
+        PrintConv => \&ConvertTimecode,
+    },
+    TCDO => {
+        Name => 'EndTimecode',
+        Groups => { 2 => 'Video' },
+        ValueConv => '$val * 1e-7',
+        PrintConv => \&ConvertTimecode,
+    },
+    VMAJ => {
+        Name => 'VegasVersionMajor',
+        Groups => { 2 => 'Video' },
+    },
+    VMIN => {
+        Name => 'VegasVersionMinor',
+        Groups => { 2 => 'Video' },
+    },
+    CMNT => {
+        Name => 'Comment',
+        Groups => { 2 => 'Video' },
+    },
+    RATE => {
+        Name => 'Rate', #? (video? units?)
+        Groups => { 2 => 'Video' },
+    },
+    STAT => {
+        Name => 'Statistics',
+        Groups => { 2 => 'Video' },
+        # ("7318 0 3.430307 1", "0 0 3500.000000 1", "7 0 3.433228 1")
+        PrintConv => [
+            '"$val frames captured"',
+            '"$val dropped"',
+            '"Data rate $val"',
+            { 0 => 'Bad', 1 => 'OK' }, # capture OK?
+        ],
+    },
+    DTIM => {
+        Name => 'DateTimeOriginal',
+        Description => 'Date/Time Original',
+        Groups => { 2 => 'Time' },
+        ValueConv => q{
+            my @v = split ' ', $val;
+            return undef unless @v == 2;
+            # get time in seconds
+            $val = 1e-7 * ($v[0] * 4294967296 + $v[1]);
+            # shift from Jan 1, 1601 to Jan 1, 1970
+            $val -= 134774 * 24 * 3600 if $val != 0;
+            return Image::ExifTool::ConvertUnixTime($val);
+        },
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
 );
 
 # Sub chunks of EXIF LIST chunk
 %Image::ExifTool::RIFF::Exif = (
     PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessChunks,
     GROUPS => { 2 => 'Audio' },
+    NOTES => 'These tags are part of the EXIF 2.3 specification for WAV audio files.',
     ever => 'ExifVersion',
     erel => 'RelatedImageFile',
     etim => { Name => 'TimeCreated', Groups => { 2 => 'Time' } },
@@ -470,11 +656,15 @@ $Image::ExifTool::RIFF::streamType = '';
         ValueConv => 'Image::ExifTool::RIFF::ConvertRIFFDate($val)',
         PrintConv => '$self->ConvertDateTime($val)',
     },
-    ISMP => 'TimeCode',
+    ISMP => 'Timecode',
     LIST_strl => {
         Name => 'Stream',
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::Stream' },
     },
+    LIST_odml => {
+        Name => 'OpenDML',
+        SubDirectory => { TagTable => 'Image::ExifTool::RIFF::OpenDML' },
+    },
 );
 
 # Sub chunks of Tdat LIST chunk (ref PH)
@@ -491,7 +681,8 @@ $Image::ExifTool::RIFF::streamType = '';
     FIRST_ENTRY => 0,
     0 => {
         Name => 'FrameRate',
-        ValueConv => '$val ? 1e6 / $val : undef',
+        # (must use RawConv because raw value used in Composite tag)
+        RawConv => '$val ? 1e6 / $val : undef',
         PrintConv => 'int($val * 1000 + 0.5) / 1000',
     },
     1 => {
@@ -534,11 +725,30 @@ $Image::ExifTool::RIFF::streamType = '';
     ],
 );
 
+# Open DML tags (ref http://www.morgan-multimedia.com/download/odmlff2.pdf)
+%Image::ExifTool::RIFF::OpenDML = (
+    PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessChunks,
+    GROUPS => { 2 => 'Video' },
+    dmlh => {
+        Name => 'ExtendedAVIHeader',
+        SubDirectory => { TagTable => 'Image::ExifTool::RIFF::ExtAVIHdr' },
+    },
+);
+
+# Extended AVI Header tags (ref http://www.morgan-multimedia.com/download/odmlff2.pdf)
+%Image::ExifTool::RIFF::ExtAVIHdr = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Video' },
+    FORMAT => 'int32u',
+    0 => 'TotalFrameCount',
+);
+
 %Image::ExifTool::RIFF::StreamHeader = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     GROUPS => { 2 => 'Video' },
     FORMAT => 'int32u',
     FIRST_ENTRY => 0,
+    PRIORITY => 0,  # so we get values from the first stream
     0 => {
         Name => 'StreamType',
         Format => 'string[4]',
@@ -550,18 +760,63 @@ $Image::ExifTool::RIFF::streamType = '';
             vids => 'Video',
         },
     },
-    1 => {
-        Name => 'Codec',
-        Format => 'string[4]',
-    },
+    1 => [
+        {
+            Name => 'AudioCodec',
+            Condition => '$Image::ExifTool::RIFF::streamType eq "auds"',
+            Format => 'string[4]',
+        },
+        {
+            Name => 'VideoCodec',
+            Condition => '$Image::ExifTool::RIFF::streamType eq "vids"',
+            Format => 'string[4]',
+        },
+        {
+            Name => 'Codec',
+            Format => 'string[4]',
+        },
+    ],
   # 2 => 'StreamFlags',
   # 3 => 'StreamPriority',
   # 3.5 => 'Language',
   # 4 => 'InitialFrames',
-  # 5 => 'Scale',
-  # 6 => 'Rate',
+    5 => [
+        {
+            Name => 'AudioSampleRate',
+            Condition => '$Image::ExifTool::RIFF::streamType eq "auds"',
+            Format => 'rational64u',
+            ValueConv => '$val ? 1/$val : 0',
+            PrintConv => 'int($val * 100 + 0.5) / 100',
+        },
+        {
+            Name => 'VideoFrameRate',
+            Condition => '$Image::ExifTool::RIFF::streamType eq "vids"',
+            Format => 'rational64u',
+            # (must use RawConv because raw value used in Composite tag)
+            RawConv => '$val ? 1/$val : undef',
+            PrintConv => 'int($val * 1000 + 0.5) / 1000',
+        },
+        {
+            Name => 'StreamSampleRate',
+            Format => 'rational64u',
+            ValueConv => '$val ? 1/$val : 0',
+            PrintConv => 'int($val * 1000 + 0.5) / 1000',
+        },
+    ],
   # 7 => 'Start',
-  # 8 => 'Length',
+    8 => [
+        {
+            Name => 'AudioSampleCount',
+            Condition => '$Image::ExifTool::RIFF::streamType eq "auds"',
+        },
+        {
+            Name => 'VideoFrameCount',
+            Condition => '$Image::ExifTool::RIFF::streamType eq "vids"',
+        },
+        {
+            Name => 'StreamSampleCount',
+        },
+    ],
   # 9 => 'SuggestedBufferSize',
     10 => {
         Name => 'Quality',
@@ -577,7 +832,10 @@ $Image::ExifTool::RIFF::streamType = '';
 %Image::ExifTool::RIFF::StreamData = ( #PH
     PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessStreamData,
     GROUPS => { 2 => 'Video' },
-    NOTES => 'This chunk contains EXIF information in FujiFilm AVI videos.',
+    NOTES => q{
+        This chunk is used to store proprietary information in AVI videos from some
+        cameras.  The first 4 characters of the data are used as the Tag ID below.
+    },
     AVIF => {
         Name => 'AVIF',
         SubDirectory => {
@@ -591,6 +849,7 @@ $Image::ExifTool::RIFF::streamType = '';
         Name => 'CasioData',
         SubDirectory => { TagTable => 'Image::ExifTool::Casio::AVI' },
     },
+    Zora => 'VendorName',   # (Samsung PL90 AVI files)
     unknown => {
         Name => 'UnknownData',
         # try to interpret unknown stream data as a string
@@ -602,10 +861,41 @@ $Image::ExifTool::RIFF::streamType = '';
 %Image::ExifTool::RIFF::Composite = (
     Duration => {
         Require => {
-            0 => 'FrameRate',
-            1 => 'FrameCount',
+            0 => 'RIFF:FrameRate',
+            1 => 'RIFF:FrameCount',
         },
-        ValueConv => '$val[0] ? $val[1] / $val[0] : undef',
+        Desire => {
+            2 => 'VideoFrameRate',
+            3 => 'VideoFrameCount',
+        },
+        # this is annoying.  Apparently (although I couldn't verify this), FrameCount
+        # in the RIFF header includes multiple video tracks if they exist (ie. with the
+        # FujiFilm REAL 3D AVI's), but the video stream information isn't reliable for
+        # some cameras (ie. Olympus FE models), so use the video stream information
+        # only if the RIFF header duration is 2 to 3 times longer
+        RawConv => q {
+            my ($dur1, $dur2);
+            $dur1 = $val[1] / $val[0] if $val[0];
+            return $dur1 unless $val[2] and $val[3];
+            $dur2 = $val[3] / $val[2];
+            my $rat = $dur1 / $dur2;
+            return ($rat > 1.9 and $rat < 3.1) ? $dur2 : $dur1;
+        },
+        PrintConv => 'ConvertDuration($val)',
+    },
+    Duration2 => {
+        Name => 'Duration',
+        Require => {
+            0 => 'RIFF:AvgBytesPerSec',
+            1 => 'FileSize',
+        },
+        Desire => {
+            # check FrameCount because this calculation only applies
+            # to audio-only files (ie. WAV)
+            2 => 'FrameCount',
+            3 => 'VideoFrameCount',
+        },
+        RawConv => '($val[0] and not ($val[2] or $val[3])) ? $val[1] / $val[0] : undef',
         PrintConv => 'ConvertDuration($val)',
     },
 );
@@ -637,6 +927,20 @@ sub ConvertRIFFDate($)
     return $val;
 }
 
+#------------------------------------------------------------------------------
+# Print time
+# Inputs: 0) time in seconds
+# Returns: time string
+sub ConvertTimecode($)
+{
+    my $val = shift;
+    my $hr = int($val / 3600);
+    $val -= $hr * 3600;
+    my $min = int($val / 60);
+    $val -= $min * 60;
+    return sprintf("%d:%.2d:%05.2f", $hr, $min, $val);
+}
+
 #------------------------------------------------------------------------------
 # Process stream data
 # Inputs: 0) ExifTool object ref, 1) dirInfo reference, 2) tag table ref
@@ -704,6 +1008,7 @@ sub ProcessChunks($$$)
     my $start = $$dirInfo{DirStart};
     my $size = $$dirInfo{DirLen};
     my $end = $start + $size;
+    my $base = $$dirInfo{Base};
 
     if ($exifTool->Options('Verbose')) {
         $exifTool->VerboseDir($$dirInfo{DirName}, 0, $size);
@@ -727,8 +1032,16 @@ sub ProcessChunks($$$)
         if ($tagInfo) {
             if ($$tagInfo{SubDirectory}) {
                 # adjust base if necessary (needed for Ricoh maker notes)
-                my $newBase = $tagInfo->{SubDirectory}->{Base};
-                $baseShift = $newBase - $$dirInfo{Base} if defined $newBase;
+                my $newBase = $tagInfo->{SubDirectory}{Base};
+                if (defined $newBase) {
+                    # different than your average Base eval...
+                    # here we use an absolute $start address
+                    $start += $base;
+                    #### eval Base ($start)
+                    $newBase = eval $newBase;
+                    $baseShift = $newBase - $base;
+                    $start -= $base;
+                }
             } elsif (not $$tagInfo{Binary}) {
                 $val = substr($$dataPt, $start, $len);
                 $val =~ s/\0+$//;   # remove trailing nulls from strings
@@ -739,7 +1052,7 @@ sub ProcessChunks($$$)
             DataPos => $$dirInfo{DataPos} - $baseShift,
             Start   => $start,
             Size    => $len,
-            Base    => $$dirInfo{Base} + $baseShift,
+            Base    => $base + $baseShift,
         );
         ++$len if $len & 0x01;  # must account for padding if odd number of bytes
         $start += $len;
@@ -756,13 +1069,13 @@ sub ProcessRIFF($$)
     my ($exifTool, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
     my ($buff, $err);
-    my %types = ( 'WAVE' => 'WAV', 'AVI ' => 'AVI' );
+    my %types = ( 'WAVE' => 'WAV', 'AVI ' => 'AVI', 'WEBP' => 'WEBP' );
     my $verbose = $exifTool->Options('Verbose');
 
     # verify this is a valid RIFF file
     return 0 unless $raf->Read($buff, 12) == 12;
     return 0 unless $buff =~ /^RIFF....(.{4})/s;
-    $exifTool->SetFileType($types{$1}); # set type to 'WAV', 'AVI' or 'RIFF'
+    $exifTool->SetFileType($types{$1}); # set type to 'WAV', 'AVI', 'WEBP' or 'RIFF'
     $Image::ExifTool::RIFF::streamType = '';    # initialize stream type
     SetByteOrder('II');
     my $tagTablePtr = GetTagTable('Image::ExifTool::RIFF::Main');
@@ -835,7 +1148,7 @@ including Windows WAV audio and AVI video files.
 
 =head1 AUTHOR
 
-Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
